@@ -108,7 +108,6 @@
 #'
 #' @examples
 #' # Load dataset
-#' library(miaViz)
 #' data("enterotype", package = "mia")
 #' tse <- enterotype
 #'
@@ -192,8 +191,6 @@ setMethod("plotRDA", signature = c(x = "SingleCellExperiment"),
             stop("reducedDim specified by 'dimred' must have at least 2 ",
                 "columns.", call. = FALSE)
         }
-        # Subset by taking only constrained axes
-        reduced_dim <- .subset_constrained_rda(reduced_dim)
         # Create an argument list. Only 2 dimensions are supported currently.
         args <- c(list(
             tse = x, dimred = dimred, reduced_dim = reduced_dim),
@@ -240,35 +237,6 @@ setMethod("plotRDA", signature = c(x = "matrix"),
     return(object)
 }
 
-# The data can include constrained and unconstrained axes. This function subsets
-# the data so that it includes only constrained axes.
-.subset_constrained_rda <- function(reduced_dim){
-    # Get only the indices of constrained ones, i.e., first set of axes.
-    # The colnames are in format, constrained_axis1, ca2, ca3..., unconstrained
-    # axis1, uca2, ...
-    comp_num <- as.numeric(gsub("\\D", "", colnames(reduced_dim)))
-    ind <- which( cumsum(comp_num == 1) <= 1 )
-    # If there were problems, it might be that the names are just arbitrary.
-    # Then take all the columns.
-    if( !(length(ind) > 0L && all(diff(ind) == 1L)) ){
-        ind <- seq_len(ncol(reduced_dim))
-    }
-    # Preserve attributes
-    attributes <- attributes(reduced_dim)
-    attributes <- attributes[ !names(attributes) %in% c("dim", "dimnames") ]
-    # Subset the data so that it includes only constrained axes
-    reduced_dim <- reduced_dim[ , ind, drop = FALSE]
-    if( "biplot" %in% names(attributes) ){
-        attributes[["biplot"]] <- attributes[["biplot"]][ , ind, drop = FALSE]
-    }
-    if( "eig" %in% names(attributes) ){
-        attributes[["eig"]] <- attributes[["eig"]][ind]
-    }
-    # Add attributes back
-    attributes(reduced_dim) <- c(attributes(reduced_dim), attributes)
-    return(reduced_dim)
-}
-
 # This function retrieves optional data that is used for creating an ellipses.
 #' @importFrom scater retrieveCellInfo
 .get_rda_ellipse_data <- function(
@@ -362,7 +330,8 @@ setMethod("plotRDA", signature = c(x = "matrix"),
     # name might be merged. Get the original variable names and
     # groups.
     if( !is.null(vector_data) ){
-        vector_data <- .get_variable_mapping_from_coldata(tse, vector_data)
+        vector_data <- .get_variable_mapping_from_coldata(
+            tse, vector_data, reduced_dim)
     }
     vars_found <- all(c("var", "levels") %in% colnames(vector_data))
     # Make the vector labels tidier. For instance, covriate name and value
@@ -406,26 +375,64 @@ setMethod("plotRDA", signature = c(x = "matrix"),
 # The RDA/CCA modifies the variable names. Those variables that are factor or
 # character i.e., groups, they get variable names that tell the variable and
 # group. This function matches those modified names with the original data.
-.get_variable_mapping_from_coldata <- function(tse, vector_data, ...){
-    # Loop over each variable in colData. Get all the possible values that they
-    # can get in RDA/CCA methods.
-    name_map <- lapply(colnames(colData(tse)), function(col){
-        # If the value is factor, get all possible values
-        if( is.factor(tse[[col]]) || is.character(tse[[col]]) ){
-            levels <- levels(as.factor(tse[[col]]))
-            name <- paste0(col, levels)
-            var <- rep(col, length(levels))
-            res <- data.frame(var, levels, name)
+.get_variable_mapping_from_coldata <- function(
+        tse, vector_data, reduced_dim, ...){
+    # Extract covariate names from RDA/CCA object
+    rda_obj <- .get_rda_attribute(reduced_dim, "obj")
+    rda_terms <- rda_obj[["terms"]]
+    factors_mat <- .get_rda_attribute(rda_terms, "factors")
+    # Build name_map from terms in the RDA/CCA model
+    name_map <- lapply(factors_mat |> ncol() |> seq_len(), function(i){
+        # Get all variables in term
+        vars_in_term <- rownames(factors_mat)[factors_mat[, i] != 0]
+        # If covariate is not interaction term
+        if( length(vars_in_term) == 1L ) {
+            # main effect
+            values <- tse[[vars_in_term]]
+            lvls <- NA_character_
+            nams <- vars_in_term
+            if( is.factor(values) || is.character(values) ){
+                lvls <- values |> as.factor() |> levels()
+                nams <- paste0(vars_in_term, lvls)
+            }
+            res <- data.frame(
+                var = vars_in_term, levels = lvls, name = nams,
+                stringsAsFactors = FALSE)
         } else{
-            # The name of numeric variables are not changed
-            res <- data.frame(var = col, levels = NA, name = col)
+            # If the term includes interaction between multiple variables
+            # Get interaction terms. Categorical values are handled differently
+            # as they create levels.
+            combos <- lapply(vars_in_term, function(var_name){
+                if( tse[[var_name]] |> is.numeric() ){
+                    temp <- var_name
+                } else{
+                    temp <- tse[[var_name]] |> unique()
+                }
+                return(temp)
+            }) |> expand.grid(stringsAsFactors = FALSE)
+            # Parse level names from values
+            level_names <- apply(combos, 1L, paste, collapse = ":")
+            # Parse interaction term name from variable names
+            var_name <- paste(vars_in_term, collapse = ":")
+            # Create final term name that was used in vegan::dbrda. It is
+            # combination of variable name and levels (if categorical)
+            name <- apply(combos, 1L, function(x){
+                temp <- mapply(function(var, val){
+                    if( tse[[var]] |> is.numeric() ) var else paste0(var, val)
+                }, vars_in_term, x)
+                temp <- temp |> paste0(collapse = ":")
+                return(temp)
+            })
+            res <- data.frame(
+                var = var_name, levels = level_names, name = name,
+                stringsAsFactors = FALSE)
         }
         return(res)
     })
     name_map <- do.call(rbind, name_map)
     # Check that all variables can be found from colData
     if( !all(rownames(vector_data) %in% name_map[["name"]]) ){
-        warning("All variables in RDA/CCA rsults must be present in ",
+        warning("All variables in RDA/CCA results must be present in ",
             "colData(x).", call. = FALSE)
     } else{
         # Add group names to vector data
@@ -476,10 +483,16 @@ setMethod("plotRDA", signature = c(x = "matrix"),
         lab <- vector_data[i, "vector_label"]
         expl_var <- round(vector_data[i, "Explained variance"]*100, 1)
         p_value <- round(vector_data[i, "Pr(>F)"], 3)
-        temp <- paste(
-            !!lab, " (", !!format(expl_var, nsmall = 1), "%, ",
-            italic("P"), " = ",
-            !!gsub("0\\.","\\.", format( p_value, nsmall = 3)), ")") |> expr()
+        # Only valid significance information is labeled. Skip NAs.
+        temp <- if( !is.na(expl_var) || !is.na(p_value) ){
+            paste(
+                !!lab, " (", !!format(expl_var, nsmall = 1), "%, ",
+                italic("P"), " = ",
+                !!gsub("0\\.","\\.", format(p_value, nsmall = 3)), ")"
+            ) |> expr()
+        } else{
+            paste(!!lab) |> expr()
+        }
         return(temp)
         }
     ) |> unlist()
@@ -540,6 +553,7 @@ setMethod("plotRDA", signature = c(x = "matrix"),
 # we can build the the plot. The idea is that the theme is similar in all
 # ordination plots.
 #' @importFrom scater plotReducedDim
+#' @importFrom ggplot2 xlab ylab
 .create_rda_baseplot <- function(
         tse, dimred, reduced_dim, ncomponents = 2L,
         add.expl.var = FALSE, expl.var = expl_var, expl_var = NULL,
@@ -569,10 +583,11 @@ setMethod("plotRDA", signature = c(x = "matrix"),
         # Convert to explained variance and take only first two components
         expl_var <- eigen_vals / sum(eigen_vals)
         expl_var <- expl_var[seq_len(ncomponents)]*100
+        expl.var <- expl_var |> round(digits = 1)
     }
     # Create argument list
     args <- c(list(object = tse, dimred = dimred, ncomponents = ncomponents,
-        colour_by = colour_by, percentVar = expl_var), list(...))
+        colour_by = colour_by, percentVar = expl.var), list(...))
     # Remove additional arguments since plotReducedDim fails if we feed
     # values that are not recognized
     remove <- names(args) %in% c(
@@ -588,6 +603,14 @@ setMethod("plotRDA", signature = c(x = "matrix"),
     # Get scatter plot with plotReducedDim --> keep theme similar between
     # ordination methods
     p <- do.call(plotReducedDim, args)
+    # Replace axis titles with names from result matrix
+    xlab <- colnames(reducedDim(tse, dimred))[[1L]]
+    ylab <- colnames(reducedDim(tse, dimred))[[2L]]
+    if( !is.null(expl.var) ){
+        xlab <- paste0(xlab, " (", expl.var[[1L]], "%)")
+        ylab <- paste0(ylab, " (", expl.var[[2L]], "%)")
+    }
+    p <- p + xlab(xlab) + ylab(ylab)
     return(p)
 }
 
@@ -636,7 +659,7 @@ setMethod("plotRDA", signature = c(x = "matrix"),
     }
     #
     data <- plot_data[["ellipse_data"]]
-    if( !is.null(data) ){
+    if( !is.null(data) && length(data) > 2L){
         xvar <- colnames(data)[[1]]
         yvar <- colnames(data)[[2]]
         colour_var <- attributes(data)[["colour_by"]]
